#include <sourcemod>
#include <sdktools>
#include <sdkhooks>
#include <vip_core>

#define PLUGIN_VERSION "1.1"
#define VIP_ST "ShowTriggers"

#pragma semicolon 1 

#define EF_NODRAW 32

int g_Offset_m_fEffects = -1;
bool g_bShowTriggers[MAXPLAYERS+1];

int g_iTransmitCount;

public Plugin myinfo =
{
	name = "[VIP] Show Triggers",
	author = "ici, Tonki_Ton",
	description = "Make trigger brushes visible.",
	version = PLUGIN_VERSION,
	url = "http://steamcommunity.com/id/1ci"
};

public void OnPluginStart()
{
	g_Offset_m_fEffects = FindSendPropInfo("CBaseEntity", "m_fEffects");
	
	if (g_Offset_m_fEffects == -1)
		SetFailState("[Show Triggers] Could not find CBaseEntity:m_fEffects");
	
	CreateConVar("showtriggers_version", PLUGIN_VERSION, "Showtriggers version", FCVAR_NOTIFY|FCVAR_REPLICATED);
	
	RegConsoleCmd("sm_showtriggers", SM_ShowTriggers, "Command to dynamically toggle trigger visibility");

	HookEvent("player_spawn", Events);
	HookEvent("player_death", Events);

	if (VIP_IsVIPLoaded())
		VIP_OnVIPLoaded();
}

public void OnPluginEnd()
{
	if (CanTestFeatures() && GetFeatureStatus(FeatureType_Native, "VIP_UnregisterFeature") == FeatureStatus_Available)
	{
		VIP_UnregisterFeature(VIP_ST);
	}
}

public void VIP_OnVIPLoaded()
{
	VIP_RegisterFeature(VIP_ST, BOOL);
}

public Action Events(Event hEvent, char[] sName, bool bDontBroadcast)
{
	int iClient = GetClientOfUserId(hEvent.GetInt("userid"));
	if (!VIP_IsClientVIP(iClient) && VIP_GetClientFeatureStatus(iClient, VIP_ST) == ENABLED)
		return;

	ClientCommand(iClient, "sm_showtriggers");
}

public Action SM_ShowTriggers(int iClient, int args)
{
	if (!iClient || !VIP_IsClientVIP(iClient))
		return Plugin_Handled;

	g_bShowTriggers[iClient] = !g_bShowTriggers[iClient];

	if (g_bShowTriggers[iClient]) {
		++g_iTransmitCount;
		PrintToChat(iClient, "Показ триггеров включен.");
	} else {
		--g_iTransmitCount;
		PrintToChat(iClient, "Показ триггеров отключен.");
	}

	transmitTriggers( g_iTransmitCount > 0 );

	return Plugin_Handled;
}

public void OnClientDisconnect_Post(int iClient)
{
	// Has this player been still using the feature before he left?
	if (g_bShowTriggers[iClient])
	{
		g_bShowTriggers[iClient] = false;
		--g_iTransmitCount;
		transmitTriggers( g_iTransmitCount > 0 );
	}
}

void transmitTriggers(bool transmit)
{
	// Hook only once
	static bool s_bHooked = false;
	
	// Have we done this before?
	if (s_bHooked == transmit)
		return;
	
	// Loop through entities
	char sBuffer[8];
	int lastEdictInUse = GetEntityCount();
	for (int entity = MaxClients+1; entity <= lastEdictInUse; ++entity)
	{
		if ( !IsValidEdict(entity) )
			continue;
		
		// Is this entity a trigger?
		GetEdictClassname(entity, sBuffer, sizeof(sBuffer));
		if (strcmp(sBuffer, "trigger") != 0)
			continue;
		
		// Is this entity's model a VBSP model?
		GetEntPropString(entity, Prop_Data, "m_ModelName", sBuffer, 2);
		if (sBuffer[0] != '*') {
			// The entity must have been created by a plugin and assigned some random model.
			// Skipping in order to avoid console spam.
			continue;
		}
		
		// Get flags
		int effectFlags = GetEntData(entity, g_Offset_m_fEffects);
		int edictFlags = GetEdictFlags(entity);
		
		// Determine whether to transmit or not
		if (transmit) {
			effectFlags &= ~EF_NODRAW;
			edictFlags &= ~FL_EDICT_DONTSEND;
		} else {
			effectFlags |= EF_NODRAW;
			edictFlags |= FL_EDICT_DONTSEND;
		}
		
		// Apply state changes
		SetEntData(entity, g_Offset_m_fEffects, effectFlags);
		ChangeEdictState(entity, g_Offset_m_fEffects);
		SetEdictFlags(entity, edictFlags);
		
		// Should we hook?
		if (transmit)
			SDKHook(entity, SDKHook_SetTransmit, Hook_SetTransmit);
		else
			SDKUnhook(entity, SDKHook_SetTransmit, Hook_SetTransmit);
	}
	s_bHooked = transmit;
}

public Action Hook_SetTransmit(int entity, int iClient)
{
	if (!g_bShowTriggers[iClient])
	{
		// I will not display myself to this iClient :(
		return Plugin_Handled;
	}
	return Plugin_Continue;
}